/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Launchy
 *
 * The Initial Developer of the Original Code is Henrik Gemal.
 * Portions created by the Initial Developer are Copyright (C) 2003-2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Henrik Gemal <launchy@gemal.dk> http://gemal.dk
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function rdfdump(rdf) {
	try {
		var conts = rdf.getAllContainers();
		for (var i=0; i<conts.length; i++) {
			walk_container(conts[i]);
		}
	} catch(err) {
		launchyShared.trace("ERR: " + err);
	}		
}
function walk_container(res) {
	var list = res.getSubNodes();
	launchyShared.trace("Apps: " + list.length);
	for (var i=0; i<list.length; i++) {
		launchyShared.trace("node: " + list[i].getSubject());
		walk_attributes(list[i]);
	}
}
function walk_attributes(node) {
	var subject = node.getSubject();
	subject.match(/urn:launchy:app:(.*)/);
	var app = RegExp.$1;
	try {
		launchyContext.addApp(node.getAttribute("http://launchy.mozdev.org/app#registry_type"), node.getAttribute("http://launchy.mozdev.org/app#registry_key"), node.getAttribute("http://launchy.mozdev.org/app#registry_entry"), app, node.getAttribute("http://launchy.mozdev.org/app#exe"), node.getAttribute("http://launchy.mozdev.org/app#type"), node.getAttribute("http://launchy.mozdev.org/app#arguments"), node.getAttribute("http://launchy.mozdev.org/app#gecko"));
	} catch(err) {
		launchyShared.trace("ERR: " + err);
	}		
}

var LaunchyContext = function() {}

LaunchyContext.prototype.findApp = function(reg, reg_sub, reg_name, app_name, app_exe, app_gecko) {

	// reg values:
	// 0: HKEY_CLASSES_ROOT
	// 1: HKEY_CURRENT_CONFIG
	// 2: HKEY_CURRENT_USER
	// 3: HKEY_LOCAL_MACHINE
	// 4: HKEY_USERS

	var reg_type = new Array("HKEY_CLASSES_ROOT", "HKEY_CURRENT_CONFIG", "HKEY_CURRENT_USER", "HKEY_LOCAL_MACHINE", "HKEY_USERS");
	launchyShared.trace("Looking for <" + app_name + "> in <" + reg_type[reg] + "> at <" + reg_sub + (reg_name?"\\"+reg_name:"") + ">" + (app_gecko?" and it's a Gecko":""));
	try {
		var tmp = LaunchyContext.winhooks.getRegistryEntry(reg, reg_sub, reg_name);
		var tmp_gecko;
		if (app_gecko && tmp) {
			tmp_gecko = tmp;
			tmp = LaunchyContext.winhooks.getRegistryEntry(reg, reg_sub + "\\" + tmp + "\\Main", "PathToExe");
		}
	} catch(ex) {
		launchyShared.trace("Unable to find " + app_name + " due to " + ex);
		return false;
	}
	if (tmp) {
		launchyShared.trace("<" + app_name + "> registry entry found");
		if (app_exe) {
			tmp += "\\" + app_exe;
		}

		var re=/\"(.*?)\" ?/;
		var x = tmp.match(re);
		if (x && x[1]) {
			tmp = x[1];
		}

		if (LaunchyContext.fu.exists(tmp)) {
			launchyShared.trace(app_name + (app_gecko ? " <" + tmp_gecko + ">" : "") + " is installed at <" + tmp + ">");
			if (app_gecko && tmp.toLowerCase().indexOf(LaunchyContext.progdir) != -1) {
				launchyShared.trace("Hide " + app_name + " since it's the running client");
				return false;
			}
			return tmp;
		} else {
			launchyShared.trace(app_name + " isn't at: <" + tmp + ">");
			return false;
		}
	} else {
		launchyShared.trace(app_name + " isn't installed or undetected");
		return false;
	}
}

LaunchyContext.prototype.createApp = function(app_name, app_exe, app_type, app_args) {
	var app = new Array();
	app.name = app_name;
	app.exe = app_exe;
	app.type = app_type;
	app.args = app_args;
	return app;
}

LaunchyContext.prototype.addApp = function(reg, reg_sub, reg_name, app_name, app_exe, app_type, app_args, app_gecko) {
	// Check for dupes
	for (i = 0; LaunchyContext.apps.length > i; i++) {
		if (LaunchyContext.apps[i].name == app_name) {
			launchyShared.trace("<" + app_name + "> is already detected");
			return;
		}
	}
	var tmp = launchyContext.findApp(reg, reg_sub, reg_name, app_name, app_exe, app_gecko);
	if (tmp) {
		LaunchyContext.apps[LaunchyContext.apps.length] = launchyContext.createApp(app_name, tmp, app_type, app_args);
	}
}

LaunchyContext.prototype.loadApps = function() {
	var chromedir = new Dir(LaunchyContext.du.getUserChromeDir());
	var file = chromedir.append("launchy.rdf");
	if (LaunchyContext.fu.exists(file)) {
		launchyShared.trace("Loading applications from RDF <" + file + ">");
		var rdf = new RDFFile(file);
		rdfdump(rdf);
	}
}

LaunchyContext.prototype.doInit = function() {
	// did Launchy already load?
	var ass = Components.classes["@mozilla.org/appshell/appShellService;1"].getService(Components.interfaces.nsIAppShellService);
	w = ass.hiddenDOMWindow;
	if (!w.launchy) {
		LaunchyContext.version = "2.0.0";
		var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		var pref = prefService.getBranch(null);
		try {
			LaunchyContext.dev = pref.getBoolPref("extensions.launchy.developer");
			launchyShared.trace("Developer information is on");
			launchyShared.trace("Launchy version <" + LaunchyContext.version + "> installed");
		} catch(ex) {
		}
	
		LaunchyContext.launchy = true;
		try {
			include(jslib_fileutils);
			include(jslib_dirutils);
			include(jslib_dir);
			include(jslib_rdf);
			include(jslib_rdffile);
			include(jslib_rdfcontainer);
			include(jslib_rdfresource);
			include(jslib_rdfmemory);
		} catch(ex) {
			launchyShared.trace("JSLib isn't installed! Launchy will be disabled!\n" + ex);
			LaunchyContext.launchy = false;
			return;
		}
	
		launchyShared.trace("JSLib version <" + JS_LIB_VERSION + "> installed");
	
		LaunchyContext.du=new DirUtils();
		LaunchyContext.progdir = LaunchyContext.du.getCurProcDir().toLowerCase();
		launchyShared.trace("Program directory is <" + LaunchyContext.progdir + ">");
	
		LaunchyContext.fu=new FileUtils();
		LaunchyContext.winhooks = Components.classes["@mozilla.org/winhooks;1"].getService(Components.interfaces.nsIWindowsRegistry);
	
		LaunchyContext.apps = new Array();
	
		if (!LaunchyContext.launchy)
			return;
	
		launchyContext.loadApps();
		launchyShared.trace("Loading applications from JS");

		// Browsers
		launchyContext.addApp(3, "Software\\mozilla.org\\Mozilla", "CurrentVersion", "Mozilla", "", 1, "", 1);
		launchyContext.addApp(3, "Software\\mozilla.org\\Mozilla", "CurrentVersion", "Mozilla", "", 1, "", 1);
		launchyContext.addApp(3, "Software\\mozilla.org\\Mozilla Firebird", "CurrentVersion", "Mozilla Firebird", "", 1, "", 1);
		launchyContext.addApp(3, "Software\\mozilla.org\\Mozilla Firefox", "CurrentVersion", "Mozilla Firefox", "", 1, "", 1);
		launchyContext.addApp(3, "Software\\mozilla.org\\Netscape", "CurrentVersion", "Netscape", "", 1, "", 1);
		launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\Opera.exe", "", "Opera", "", 1, "", 0);
		launchyContext.addApp(2, "Software\\K-Meleon\\K-Meleon\\General", "InstallDir", "K-Meleon", "k-meleon.exe", 1, "", 0);
		launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\MYIE2", "DisplayIcon", "MyIE2", "", 1, "", 0);
		launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\IEXPLORE.EXE", "", "Internet Explorer", "", 1, "", 0);
	
		// Mail clients
		launchyContext.addApp(3, "Software\\mozilla.org\\Mozilla", "CurrentVersion", "Mozilla Mail", "", 2, ["-compose"], 1);
		launchyContext.addApp(3, "Software\\mozilla.org\\Mozilla Thunderbird", "CurrentVersion", "Mozilla Thunderbird", "", 2, ["-compose"], 1);
		launchyContext.addApp(2, "Software\\Pegasus Mail\\BaseDir", "", "Pegasus Mail", "wsendto.exe", 2, "", 0);
		launchyContext.addApp(3, "Software\\Lotus\\Notes", "Path", "Lotus Notes", "notes.exe", 2, "", 0);
		launchyContext.addApp(2, "Software\\Qualcomm\\Eudora\\CommandLine", "current", "Eudora", "", 2, "", 0);
		launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\OUTLOOK.EXE", "", "Outlook", "", 2, ["-c", "IPM.Note", "/m"], 0);
		launchyContext.addApp(2, "Software\\RIT\\The Bat!", "EXE path", "The Bat!", "", 2, "", 0);
		launchyContext.addApp(3, "Software\\IncrediMail", "InstallationPath", "IncrediMail", "bin\\IncMail.exe", 2, ["/c", "/n", "/mailto:#1"] , 0);
	
		// Media clients
		launchyContext.addApp(2, "Software\\BST\\bsplayerv1", "AppPath", "BSPlayer", "", 3, "", 0);
		launchyContext.addApp(3, "Software\\Clients\\Media\\iTunes\\shell\\open\\command", "", "iTunes", "", 3, ["/url"], 0);
		launchyContext.addApp(2, "Software\\Winamp", "", "Winamp", "winamp.exe", 3, "", 0);
		launchyContext.addApp(3, "Software\\Microsoft\\MediaPlayer", "Installation Directory", "Windows Media Player", "wmplayer.exe", 3, "", 0);
	
		// launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\UEDIT32.exe", "", "UltraEdit", "", 4, "", 0);
	
		// Special cased applications
		var tmp = launchyContext.findApp(3, "Software\\Microsoft\\WAB\\Export\\CSV", "DLL", "Outlook Express");
		if (tmp) {
			var re=/wabimp\.dll/i;
			tmp = tmp.replace(re, "msimn.exe");
			LaunchyContext.apps[LaunchyContext.apps.length] = launchyContext.createApp("Outlook Express", tmp, 2, ["/mailurl:#1"]);
		}

		w.launchy = LaunchyContext;
	} else {
		LaunchyContext = w.launchy;
	}
}

LaunchyContext.prototype.doContext = function(i) {
	if (!LaunchyContext.launchy) {
		document.getElementById(i + "-disabled").hidden = false;
		return;
	}
	if (!LaunchyContext.apps.length) {
		document.getElementById(i + "-none").hidden = false;
		return;
	}

	var menuid = document.getElementById(i);
  var children = menuid.childNodes;
  for (var j = 0; j < children.length; j++) {
		menuid.removeChild(children[j]);
	}

	var label;
	var tooltip;

	// right click on page
	if (!gContextMenu.onLink && !gContextMenu.onMailtoLink) {
		label = document.getElementById("bundle_launchy").getString("launchy-page-label");
		tooltip = document.getElementById("bundle_launchy").getString("launchy-page-tooltip");
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 1) {
				var menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\");");
				menuid.appendChild(menuitem);
			}
		}
	}

	// right click on link
	else if (gContextMenu.onLink && !gContextMenu.onMailtoLink) {
		label = document.getElementById("bundle_launchy").getString("launchy-link-label");
		tooltip = document.getElementById("bundle_launchy").getString("launchy-link-tooltip");
		var seperator = 0;
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 1) {
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
				menuid.appendChild(menuitem);
				seperator = 1;
			}
		}
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 3) {
				if (seperator == 1) {
					menuitem = document.createElement("menuseparator");
					menuid.appendChild(menuitem);
					seperator++;
				}
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
				menuid.appendChild(menuitem);
			}
		}
	}

	// right click on mailto link
	else if (gContextMenu.onMailtoLink) {
		label = document.getElementById("bundle_launchy").getString("launchy-mail-label");
		tooltip = document.getElementById("bundle_launchy").getString("launchy-mail-tooltip");
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 2) {
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
				menuid.appendChild(menuitem);
			}
		}
	}
}

LaunchyContext.prototype.doApp = function(app_entry, on_link) {
	var app_exe = LaunchyContext.apps[app_entry].exe;

	var url;
	if (on_link) {
		url = gContextMenu.link;
	} else {
		url = document.commandDispatcher.focusedWindow.document.location.href;
	}

	var xargs = new Array();
	if (LaunchyContext.apps[app_entry].args)
		xargs = LaunchyContext.apps[app_entry].args.join(',').split(',');

	var u = false;
	for (var i=0; i < xargs.length; i++) {
		var tmp = launchyShared.replaceInsert(xargs[i], 1, url)
		if (tmp != xargs[i]) {
			xargs[i] = tmp;
			u = true;
		}
	}
	if (!u)
		xargs[xargs.length] = url;

	launchyShared.trace("About to run:\n" + app_exe + " " + xargs.join(" "));
	LaunchyContext.fu.run(app_exe, xargs);
}

// create the object
const launchyContext = new LaunchyContext();
launchyContext.doInit();
